CurrencyTracker = LibStub("AceAddon-3.0"):NewAddon("CurrencyTracker", "AceConsole-3.0", "AceEvent-3.0")

--[[
	The options table, which is passed to the Ace Generator, that creates a blizzard
		options pane inside the the interface/addons window pane (in-game)
--]]
local options = {
	name = "CurrencyTracker",
	handler = CurrencyTracker,
	type = "group",
	args = {
		post = {
			name = "Post",
			desc = "Posts all currency information for this character",
			type = "execute",
			func = "PostCurrency"
		},
		display = {
			name = "Display",
			desc = "Display all currency information available",
			type = "execute",
			func = "DisplayCurrency"
		}
	}
}

--[[
	This defaults table, creates the two default tables that the addon will be working
		with.  It's global for future implemenations of cross realm gold and currency
		tracking.  It contains a currency table and a totals table which act as two
		dictionaries for each individual character and all their relevant curriences.
		(Relavant currencies determined by the CT_currencyIDs table defined below)
--]]
local defaults = {
	global = {
		currency = {
		},
		totals = {
		}
	}
}

--[[
	This array determines all the currencies the addon will track.
	The four major curriences are permanently defined on every character,
		whether or not they contain that currency or not.  Where as other
		currencies are unlocked as they're achieved.
		(Major curriences included Honor Points, Conquest Points, Justice Points,
		and Valor Points.)
--]]
local CT_currencyIDs = {
	392, -- Honor Points
	390, -- Conquest Points
	395, -- Justice Points
	396, -- Valor Points
	391, -- Tol Barad Commendation
}

--[[
	The OnInitialize function is called when the first time an addon is ran.  It creates
		an options table using the built in blizzard interface, and sets the database defaults.
	It also updates the WoW slash commands with the two slash commands associated with the addon
		(/ct and /currencytracker)
--]]
function CurrencyTracker:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("CurrencyTrackerDB", defaults, "Default")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CurrencyTracker", options, {"currencytracker", "ct"})
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CurrencyTracker",  "CurrencyTracker")
	self:RegisterChatCommand("ct", "ChatCommand")
	self:RegisterChatCommand("currencytracker", "ChatCommand")
end

--[[
	
--]]
function CurrencyTracker:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(CurrencyTracker, "ct", "CurrencyTracker", input)
	end
end

--[[
	OnEnable is a function used to track events in game, we're looking for the event that
		corresponds to the last time we're able to save variables in our database.
	This will in the future provide for the automation of when you log out your curriences
		will be updated with their newest known values before logging to keep the database
		synced with your in-game curriences.
--]]
function CurrencyTracker:OnEnable()
	self:RegisterEvent("PLAYER_LOGOUT")
end

--[[
	Should be the event we're tracking right before we're able to save our db variables,
		but as of right now appears to not be working so the automation is disabled.
--]]
function CurrencyTracker:PLAYER_LOGOUT()
	--CurrencyTracker:PostCurrency()
end

--[[
	Blank function, but for future implementaions we may be using this to clear db
		values to allow for a simple way of clearing the db values.
--]]
function CurrencyTracker:OnDisable()

end

--[[
	The PostCurrency function updates the db with the current values of the player's
		currencies. Uses a hash table dictionary setup to allow for each character to
		have a unique set of currency names and values.
	We hardcode in the value of gold, as it doesn't have a money title or name
		associated with it.
--]]
function CurrencyTracker:PostCurrency()
	local charName = UnitName("player")
	t = {}
	
	t["Gold"] = GetMoney()
	for i = 1, table.getn(CT_currencyIDs) do
		currencyName, currencyValue = GetCurrencyInfo(CT_currencyIDs[i])
		t[currencyName] = currencyValue	
	end
	
	self.db.global.currency[charName] = t
	
	local goldTotal = 0
	for k,v in pairs(self.db.global.currency) do
		for k2,v2 in pairs(v) do
			if k2 == "Gold" then
				goldTotal = goldTotal + v2
			end
		end
	end
	
	self.db.global.totals["Total Gold"] = goldTotal
	
	print("Posted Character Information")
end

--[[
	The DisplayCurrency function prints all the information contained in the
		addon's currency database. This includes the character's name, all
		currencies the character has values of more than 0, as well as the
		actual values of those currencies who's values are more than 0.
		(If the value is 0 the currency is not printed).
--]]
function CurrencyTracker:DisplayCurrency()
	for k,v in pairs(self.db.global.currency) do
		self:Print(k);
		for k2,v2 in pairs(v) do
			if k2 == "Gold" then
				print(k2, GetCoinTextureString(v2))
			elseif k2 == "" then
				--if no currency is named do nothing
			else
				print(k2, v2)
			end
		end
	end
	
	for k,v in pairs(self.db.global.totals) do
		print(k, GetCoinTextureString(v));
	end
end